@@ -18,7 +18,6 @@
 import os
 import time
 import tarfile
-import zipfile
 import shutil
 import logging
 import shlex
@@ -29,6 +28,8 @@ import sys
 # required for compression callout (FIXME: move to policy?)
 from subprocess import Popen, PIPE
 
+from sos.utilities import sos_get_command_output, is_executable
+
 try:
     import selinux
 except ImportError:
@@ -98,6 +99,12 @@ class Archive(object):
         to be included in the generated archive."""
         raise NotImplementedError
 
+    def name_max(self):
+        """Return the maximum file name length this archive can support.
+        This is the lesser of the name length limit of the archive
+        format and any temporary file system based cache."""
+        raise NotImplementedError
+
     def get_archive_path(self):
         """Return a string representing the path to the temporary
         archive. For archive classes that implement in-line handling
@@ -175,6 +182,8 @@ class FileCacheArchive(Archive):
         dest = self.dest_path(dest)
         self._check_path(dest)
         f = codecs.open(dest, 'w', encoding='utf-8')
+        if isinstance(content, bytes):
+            content = content.decode('utf8', 'ignore')
         f.write(content)
         if os.path.exists(src):
             try:
@@ -188,7 +197,7 @@ class FileCacheArchive(Archive):
     def add_link(self, source, link_name):
         dest = self.dest_path(link_name)
         self._check_path(dest)
-        if not os.path.exists(dest):
+        if not os.path.lexists(dest):
             os.symlink(source, dest)
         self.log_debug("added symlink at '%s' to '%s' in FileCacheArchive '%s'"
                        % (dest, source, self._archive_root))
@@ -206,6 +215,13 @@ class FileCacheArchive(Archive):
     def _makedirs(self, path, mode=0o700):
         os.makedirs(path, mode)
 
+    def name_max(self):
+        if 'PC_NAME_MAX' in os.pathconf_names:
+            pc_name_max = os.pathconf_names['PC_NAME_MAX']
+            return os.pathconf(self._archive_root, pc_name_max)
+        else:
+            return 255
+
     def get_tmp_dir(self):
         return self._archive_root
 
@@ -232,7 +248,12 @@ class FileCacheArchive(Archive):
         self.log_info("built archive at '%s' (size=%d)" % (self._archive_name,
                       os.stat(self._archive_name).st_size))
         self.method = method
-        return self._compress()
+        try:
+            return self._compress()
+        except Exception as e:
+            exp_msg = "An error occurred compressing the archive: "
+            self.log_error("%s %s" % (exp_msg, e))
+            return self.name()
 
 
 # Compatibility version of the tarfile.TarFile class. This exists to allow
@@ -355,6 +376,11 @@ class TarFileArchive(FileCacheArchive):
     def name(self):
         return "%s.%s" % (self._name, self._suffix)
 
+    def name_max(self):
+        # GNU Tar format supports unlimited file name length. Just return
+        # the limit of the underlying FileCacheArchive.
+        return super(TarFileArchive, self).name_max()
+
     def _build_archive(self):
         # python2.6 TarFile lacks the filter parameter
         if not six.PY3 and sys.version_info[1] < 7:
@@ -368,99 +394,34 @@ class TarFileArchive(FileCacheArchive):
         tar.close()
 
     def _compress(self):
-        methods = ['xz', 'bzip2', 'gzip']
+        methods = []
+        # Make sure that valid compression commands exist.
+        for method in ['xz', 'bzip2', 'gzip']:
+            if is_executable(method):
+                methods.append(method)
+            else:
+                self.log_error("\"%s\" command not found." % method)
         if self.method in methods:
             methods = [self.method]
 
-        last_error = Exception("compression failed for an unknown reason")
-
+        exp_msg = "No compression utilities found."
+        last_error = Exception(exp_msg)
         for cmd in methods:
             suffix = "." + cmd.replace('ip', '')
             # use fast compression if using xz or bz2
             if cmd != "gzip":
                 cmd = "%s -1" % cmd
             try:
-                command = shlex.split("%s %s" % (cmd, self.name()))
-                p = Popen(command,
-                          stdout=PIPE,
-                          stderr=PIPE,
-                          bufsize=-1,
-                          close_fds=True)
-                stdout, stderr = p.communicate()
-                if stdout:
-                    self.log_info(stdout.decode('utf-8'))
-                if stderr:
-                    self.log_error(stderr.decode('utf-8'))
+                r = sos_get_command_output("%s %s" % (cmd, self.name()))
+
+                if r['status']:
+                    self.log_info(r['output'])
+
                 self._suffix += suffix
                 return self.name()
+
             except Exception as e:
                 last_error = e
         raise last_error
 
-
-class ZipFileArchive(Archive):
-    """ archive class using python ZipFile to create zip archives """
-
-    def __init__(self, name):
-        self._name = name
-        try:
-            import zlib
-            assert zlib
-            self.compression = zipfile.ZIP_DEFLATED
-        except:
-            self.compression = zipfile.ZIP_STORED
-
-        self.zipfile = zipfile.ZipFile(self.name(), mode="w",
-                                       compression=self.compression)
-
-    def name(self):
-        return "%s.zip" % self._name
-
-    def finalize(self, method):
-        super(ZipFileArchive, self).finalize(method)
-        return self.name()
-
-    def add_file(self, src, dest=None):
-        src = str(src)
-        if dest:
-            dest = str(dest)
-
-        if os.path.isdir(src):
-            # We may not need, this, but if we do I only want to do it
-            # one time
-            regex = re.compile(r"^" + src)
-            for path, dirnames, filenames in os.walk(src):
-                for filename in filenames:
-                    filename = "/".join((path, filename))
-                    if dest:
-                        self.zipfile.write(filename, re.sub(regex, dest,
-                                                            filename))
-                    else:
-                        self.zipfile.write(filename)
-        else:
-            if dest:
-                self.zipfile.write(src, dest)
-            else:
-                self.zipfile.write(src)
-
-    def add_string(self, content, dest):
-        info = zipfile.ZipInfo(dest,
-                               date_time=time.localtime(time.time()))
-        info.compress_type = self.compression
-        info.external_attr = 0o400 << long(16)
-        self.zipfile.writestr(info, content)
-
-    def open_file(self, name):
-        try:
-            self.zipfile.close()
-            self.zipfile = zipfile.ZipFile(self.name(), mode="r")
-            file_obj = self.zipfile.open(name)
-            return file_obj
-        finally:
-            self.zipfile.close()
-            self.zipfile = zipfile.ZipFile(self.name(), mode="a")
-
-    def close(self):
-        self.zipfile.close()
-
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
