@@ -25,16 +25,32 @@ class Apache(Plugin):
         ("log", "gathers all apache logs", "slow", False)
     ]
 
+    def setup(self):
+        # collect list of installed modules
+        self.add_cmd_output(["apachectl -M"])
+
 
 class RedHatApache(Apache, RedHatPlugin):
-    files = ('/etc/httpd/conf/httpd.conf',)
+    files = (
+        '/etc/httpd/conf/httpd.conf',
+        '/etc/httpd22/conf/httpd.conf',
+        '/etc/httpd24/conf/httpd.conf'
+    )
 
     def setup(self):
         super(RedHatApache, self).setup()
 
         self.add_copy_spec([
             "/etc/httpd/conf/httpd.conf",
-            "/etc/httpd/conf.d/*.conf"
+            "/etc/httpd/conf.d/*.conf",
+            "/etc/httpd/conf.modules.d/*.conf",
+            # JBoss Enterprise Web Server 2.x
+            "/etc/httpd22/conf/httpd.conf",
+            "/etc/httpd22/conf.d/*.conf",
+            # Red Hat JBoss Web Server 3.x
+            "/etc/httpd24/conf/httpd.conf",
+            "/etc/httpd24/conf.d/*.conf",
+            "/etc/httpd24/conf.modules.d/*.conf",
         ])
 
         self.add_forbidden_path("/etc/httpd/conf/password.conf")
@@ -44,8 +60,24 @@ class RedHatApache(Apache, RedHatPlugin):
         self.add_copy_spec_limit("/var/log/httpd/error_log", 5)
         self.add_copy_spec_limit("/var/log/httpd/ssl_access_log", 5)
         self.add_copy_spec_limit("/var/log/httpd/ssl_error_log", 5)
+        # JBoss Enterprise Web Server 2.x
+        self.add_copy_spec_limit("/var/log/httpd22/access_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd22/error_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd22/ssl_access_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd22/ssl_error_log", 5)
+        # Red Hat JBoss Web Server 3.x
+        self.add_copy_spec_limit("/var/log/httpd24/access_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd24/error_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd24/ssl_access_log", 5)
+        self.add_copy_spec_limit("/var/log/httpd24/ssl_error_log", 5)
         if self.get_option("log"):
-            self.add_copy_spec("/var/log/httpd/*")
+            self.add_copy_spec([
+                "/var/log/httpd/*",
+                # JBoss Enterprise Web Server 2.x
+                "/var/log/httpd22/*",
+                # Red Hat JBoss Web Server 3.x
+                "/var/log/httpd24/*"
+            ])
 
 
 class DebianApache(Apache, DebianPlugin, UbuntuPlugin):
@@ -64,4 +96,4 @@ class DebianApache(Apache, DebianPlugin, UbuntuPlugin):
         if self.get_option("log"):
             self.add_copy_spec("/var/log/apache2/*")
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
