@@ -24,11 +24,10 @@ class Block(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
     profiles = ('storage', 'hardware')
 
     def setup(self):
-        self.add_copy_spec("/proc/partitions")
-
         self.add_cmd_output([
             "lsblk",
             "blkid -c /dev/null",
+            "blockdev --report",
             "ls -lanR /dev",
             "ls -lanR /sys/block"
         ])
@@ -36,7 +35,10 @@ class Block(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
         # legacy location for non-/run distributions
         self.add_copy_spec([
             "/etc/blkid.tab",
-            "/run/blkid/blkid.tab"
+            "/run/blkid/blkid.tab",
+            "/proc/partitions",
+            "/proc/diskstats",
+            "/sys/block/*/queue/scheduler"
         ])
 
         if os.path.isdir("/sys/block"):
@@ -46,8 +48,8 @@ class Block(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
                 disk_path = os.path.join('/dev/', disk)
                 self.add_cmd_output([
                     "udevadm info -ap /sys/block/%s" % (disk),
-                    "parted -s %s print" % (disk_path),
+                    "parted -s %s unit s print" % (disk_path),
                     "fdisk -l %s" % disk_path
                 ])
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
