@@ -33,13 +33,20 @@ class Corosync(Plugin):
             "corosync-quorumtool -l",
             "corosync-quorumtool -s",
             "corosync-cpgtool",
+            "corosync-cfgtool -s",
+            "corosync-blackbox",
             "corosync-objctl -a",
-            "corosync-fplay",
-            "corosync-objctl -w runtime.blackbox.dump_state=$(date +\%s)",
-            "corosync-objctl -w runtime.blackbox.dump_flight_data=$(date +\%s)"
+            "corosync-cmapctl"
         ])
         self.call_ext_prog("killall -USR2 corosync")
 
+    def postproc(self):
+        self.do_cmd_output_sub(
+            "corosync-objctl",
+            r"(.*fence.*\.passwd=)(.*)",
+            r"\1******"
+        )
+
 
 class RedHatCorosync(Corosync, RedHatPlugin):
 
@@ -54,4 +61,4 @@ class DebianCorosync(Corosync, DebianPlugin, UbuntuPlugin):
 
     files = ('/usr/sbin/corosync',)
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
