@@ -19,7 +19,7 @@ class HardwareTestSuite(Plugin, RedHatPlugin):
     """Red Hat Hardware Test Suite
     """
 
-    plugin_name = 'hardwaretestsuite'
+    plugin_name = 'hts'
     profiles = ('debug',)
 
     def setup(self):
@@ -28,4 +28,4 @@ class HardwareTestSuite(Plugin, RedHatPlugin):
             "/var/hts"
         ])
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
