@@ -51,7 +51,7 @@ class PostgreSQL(Plugin):
         # is no need to save and restore environment variables if the user
         # decided to pass the password on the command line.
         if self.get_option("password") is not False:
-            os.environ["PGPASSWORD"] = self.get_option("password")
+            os.environ["PGPASSWORD"] = str(self.get_option("password"))
 
         if self.get_option("dbhost"):
             cmd = "pg_dump -U %s -h %s -p %s -w -f %s -F t %s" % (
@@ -92,13 +92,6 @@ class PostgreSQL(Plugin):
                 self.add_alert(
                     "WARN: password must be supplied to dump a database."
                 )
-        else:
-            self.soslog.warning(
-                "dbname must be supplied to dump a database."
-            )
-            self.add_alert(
-                "WARN: dbname must be supplied to dump a database."
-            )
 
     def postproc(self):
         import shutil
@@ -153,4 +146,4 @@ class DebianPostgreSQL(PostgreSQL, DebianPlugin, UbuntuPlugin):
         ])
 
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
