@@ -37,18 +37,33 @@ class Rpm(Plugin, RedHatPlugin):
     def setup(self):
         self.add_copy_spec("/var/log/rpmpkgs")
 
-        if self.get_option("rpmq"):
-            query_fmt = '"%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}~~'
-            query_fmt = query_fmt + '%{INSTALLTIME:date}\t%{INSTALLTIME}\t'
-            query_fmt = query_fmt + '%{VENDOR}\t%{BUILDHOST}\t'
-            query_fmt = query_fmt + '%{SIGPGP}\t%{SIGPGP:pgpsig}\n"'
-            rpmq_cmd = "rpm --nodigest -qa --qf=%s" % query_fmt
-            filter_cmd = 'awk -F "~~" ' \
-                '"{printf \\"%-59s %s\\n\\",\$1,\$2}"|sort'
-            shell_cmd = "sh -c '%s'" % (rpmq_cmd + "|" + filter_cmd)
-            self.add_cmd_output(shell_cmd, root_symlink="installed-rpms")
+        def add_rpm_cmd(query_fmt, filter_cmd, symlink, suggest):
+            rpmq_cmd = 'rpm --nodigest -qa --qf=%s' % query_fmt
+            shell_cmd = rpmq_cmd
+            if filter_cmd:
+                shell_cmd = "sh -c '%s'" % (rpmq_cmd + "|" + filter_cmd)
+            self.add_cmd_output(shell_cmd, root_symlink=symlink,
+                                suggest_filename=suggest)
 
-        if self.get_option("verify"):
+        if self.get_option("rpmq"):
+            # basic installed-rpms
+            query_fmt = '"%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}~~'
+            query_fmt = query_fmt + '%{INSTALLTIME:date}\n"'
+
+            filter_cmd = 'awk -F "~~" ' \
+                '"{printf \\"%-59s %s\\n\\",\$1,\$2}"|sort -f'
+
+            add_rpm_cmd(query_fmt, filter_cmd, "installed-rpms", None)
+
+            # extended package data
+            query_fmt = '"%{NAME}-%{VERSION}-%{RELEASE}.%{ARCH}\\t'
+            query_fmt = query_fmt + '%{INSTALLTIME:date}\\t%{INSTALLTIME}\\t'
+            query_fmt = query_fmt + '%{VENDOR}\\t%{BUILDHOST}\\t'
+            query_fmt = query_fmt + '%{SIGPGP}\\t%{SIGPGP:pgpsig}\\n"'
+
+            add_rpm_cmd(query_fmt, None, None, "package-data")
+
+        if self.get_option("verify") or self.get_option("rpmva"):
             if self.get_option("rpmva"):
                 self.add_cmd_output("rpm -Va", root_symlink="rpm-Va",
                                     timeout=180)
@@ -65,4 +80,4 @@ class Rpm(Plugin, RedHatPlugin):
                 self.add_cmd_output("rpm -V %s" % verify_pkgs)
 
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
