@@ -14,10 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-from sos.plugins import Plugin, RedHatPlugin
+from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
 
 
-class S390(Plugin, RedHatPlugin):
+class S390(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
     """IBM S/390
     """
 
@@ -27,7 +27,7 @@ class S390(Plugin, RedHatPlugin):
     # Check for s390 arch goes here
 
     def check_enabled(self):
-        return (self.policy().get_arch() == "s390")
+        return ("s390" in self.policy().get_arch())
 
     # Gather s390 specific information
 
@@ -71,4 +71,4 @@ class S390(Plugin, RedHatPlugin):
             ])
 
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
