@@ -32,24 +32,27 @@ from sos.utilities import ImporterHelper
 from stat import ST_UID, ST_GID, ST_MODE, ST_CTIME, ST_ATIME, ST_MTIME, S_IMODE
 from time import strftime, localtime
 from collections import deque
+from shutil import rmtree
 import tempfile
+import hashlib
 
 from sos import _sos as _
 from sos import __version__
 import sos.policies
-from sos.archive import TarFileArchive, ZipFileArchive
+from sos.archive import TarFileArchive
 from sos.reporting import (Report, Section, Command, CopiedFile, CreatedFile,
                            Alert, Note, PlainTextReport)
 
 # PYCOMPAT
 import six
 from six.moves import zip, input
-if six.PY3:
-    from configparser import ConfigParser
-else:
-    from ConfigParser import ConfigParser
 from six import print_
 
+if six.PY3:
+    from configparser import ConfigParser, ParsingError, Error
+else:
+    from ConfigParser import ConfigParser, ParsingError, Error
+
 # file system errors that should terminate a run
 fatal_fs_errors = (errno.ENOSPC, errno.EROFS)
 
@@ -99,7 +102,9 @@ class TempFileUtil(object):
 
 
 class OptionParserExtended(OptionParser):
+
     """ Show examples """
+
     def print_help(self, out=sys.stdout):
         """ Prints help content including examples """
         OptionParser.print_help(self, out)
@@ -116,6 +121,7 @@ class OptionParserExtended(OptionParser):
 
 
 class SosOption(Option):
+
     """Allow to specify comma delimited list of plugins"""
     ACTIONS = Option.ACTIONS + ("extend",)
     STORE_ACTIONS = Option.STORE_ACTIONS + ("extend",)
@@ -135,6 +141,7 @@ class SosOption(Option):
 
 
 class XmlReport(object):
+
     """ Report build class """
 
     def __init__(self):
@@ -220,6 +227,10 @@ class XmlReport(object):
         outf.close()
 
 
+# valid modes for --chroot
+chroot_modes = ["auto", "always", "never"]
+
+
 class SoSOptions(object):
     _list_plugins = False
     _noplugins = []
@@ -241,7 +252,9 @@ class SoSOptions(object):
     _list_profiles = False
     _config_file = ""
     _tmp_dir = ""
-    _report = True
+    _noreport = False
+    _sysroot = None
+    _chroot = 'auto'
     _compression_type = 'auto'
 
     _options = None
@@ -255,7 +268,7 @@ class SoSOptions(object):
     def _check_options_initialized(self):
         if self._options is not None:
             raise ValueError("SoSOptions object already initialized "
-                             + "from command line")
+                             "from command line")
 
     @property
     def list_plugins(self):
@@ -281,6 +294,16 @@ class SoSOptions(object):
         self._check_options_initialized()
         self._noplugins = value
 
+    @property
+    def experimental(self):
+        if self._options is not None:
+            return self._options.experimental
+
+    @experimental.setter
+    def experimental(self, value):
+        self._check_options_initialized()
+        self._experimental = value
+
     @property
     def enableplugins(self):
         if self._options is not None:
@@ -501,17 +524,43 @@ class SoSOptions(object):
         self._tmp_dir = value
 
     @property
-    def report(self):
+    def noreport(self):
         if self._options is not None:
-            return self._options.report
-        return self._report
+            return self._options.noreport
+        return self._noreport
 
-    @report.setter
-    def report(self, value):
+    @noreport.setter
+    def noreport(self, value):
         self._check_options_initialized()
         if not isinstance(value, bool):
-            raise TypeError("SoSOptions.report expects a boolean")
-        self._report = value
+            raise TypeError("SoSOptions.noreport expects a boolean")
+        self._noreport = value
+
+    @property
+    def sysroot(self):
+        if self._options is not None:
+            return self._options.sysroot
+        return self._sysroot
+
+    @sysroot.setter
+    def sysroot(self, value):
+        self._check_options_initialized()
+        self._sysroot = value
+
+    @property
+    def chroot(self):
+        if self._options is not None:
+            return self._options.chroot
+        return self._chroot
+
+    @chroot.setter
+    def chroot(self, value):
+        self._check_options_initialized()
+        if value not in chroot_modes:
+            msg = "SoSOptions.chroot '%s' is not a valid chroot mode: "
+            msg += "('auto', 'always', 'never')"
+            raise ValueError(msg % value)
+        self._chroot = value
 
     @property
     def compression_type(self):
@@ -534,6 +583,9 @@ class SoSOptions(object):
         parser.add_option("-n", "--skip-plugins", action="extend",
                           dest="noplugins", type="string",
                           help="disable these plugins", default=deque())
+        parser.add_option("--experimental", action="store_true",
+                          dest="experimental", default=False,
+                          help="enable experimental plugins")
         parser.add_option("-e", "--enable-plugins", action="extend",
                           dest="enableplugins", type="string",
                           help="enable these plugins", default=deque())
@@ -584,7 +636,9 @@ class SoSOptions(object):
                           dest="profiles", type="string", default=deque(),
                           help="enable plugins selected by the given profiles")
         parser.add_option("--list-profiles", action="store_true",
-                          dest="list_profiles", default=False)
+                          dest="list_profiles", default=False,
+                          help="display a list of available profiles and "
+                               "plugins that they include")
         parser.add_option("--name", action="store",
                           dest="customer_name",
                           help="specify report name")
@@ -596,10 +650,17 @@ class SoSOptions(object):
                           help="specify alternate temporary directory",
                           default=None)
         parser.add_option("--no-report", action="store_true",
-                          dest="report",
-                          help="Disable HTML/XML reporting", default=False)
+                          dest="noreport",
+                          help="disable HTML/XML reporting", default=False)
+        parser.add_option("-s", "--sysroot", action="store", dest="sysroot",
+                          help="system root directory path (default='/')",
+                          default=None)
+        parser.add_option("-c", "--chroot", action="store", dest="chroot",
+                          help="chroot executed commands to SYSROOT "
+                               "[auto, always, never] (default=auto)",
+                               default="auto")
         parser.add_option("-z", "--compression-type", dest="compression_type",
-                          help="compression technology to use [auto, zip, "
+                          help="compression technology to use [auto, "
                                "gzip, bzip2, xz] (default=auto)",
                           default="auto")
 
@@ -607,6 +668,7 @@ class SoSOptions(object):
 
 
 class SoSReport(object):
+
     """The main sosreport class"""
 
     def __init__(self, args):
@@ -618,6 +680,8 @@ class SoSReport(object):
         self.archive = None
         self.tempfile_util = None
         self._args = args
+        self.sysroot = "/"
+        self.sys_tmp = None
 
         try:
             import signal
@@ -630,26 +694,53 @@ class SoSReport(object):
         self._read_config()
 
         try:
-            self.policy = sos.policies.load()
+            self.policy = sos.policies.load(sysroot=self.opts.sysroot)
         except KeyboardInterrupt:
             self._exit(0)
 
         self._is_root = self.policy.is_root()
 
-        self.tmpdir = os.path.abspath(
-            self.policy.get_tmp_dir(self.opts.tmp_dir))
-        if not os.path.isdir(self.tmpdir) \
-                or not os.access(self.tmpdir, os.W_OK):
+        # system temporary directory to use
+        tmp = os.path.abspath(self.policy.get_tmp_dir(self.opts.tmp_dir))
+
+        if not os.path.isdir(tmp) \
+                or not os.access(tmp, os.W_OK):
+            msg = "temporary directory %s " % tmp
+            msg += "does not exist or is not writable\n"
             # write directly to stderr as logging is not initialised yet
-            sys.stderr.write("temporary directory %s " % self.tmpdir
-                             + "does not exist or is not writable\n")
+            sys.stderr.write(msg)
             self._exit(1)
+
+        self.sys_tmp = tmp
+
+        # our (private) temporary directory
+        self.tmpdir = tempfile.mkdtemp(prefix="sos.", dir=self.sys_tmp)
         self.tempfile_util = TempFileUtil(self.tmpdir)
+
         self._set_directories()
 
+        self._setup_logging()
+
+        msg = "default"
+        host_sysroot = self.policy.host_sysroot()
+        # set alternate system root directory
+        if self.opts.sysroot:
+            msg = "cmdline"
+            self.sysroot = self.opts.sysroot
+        elif self.policy.in_container() and host_sysroot != os.sep:
+            msg = "policy"
+            self.sysroot = host_sysroot
+        self.soslog.debug("set sysroot to '%s' (%s)" % (self.sysroot, msg))
+
+        if self.opts.chroot not in chroot_modes:
+            self.soslog.error("invalid chroot mode: %s" % self.opts.chroot)
+            logging.shutdown()
+            self.tempfile_util.clean()
+            self._exit(1)
+
     def print_header(self):
         self.ui_log.info("\n%s\n" % _("sosreport (version %s)" %
-                         (__version__,)))
+                                      (__version__,)))
 
     def get_commons(self):
         return {
@@ -659,12 +750,13 @@ class SoSReport(object):
             'tmpdir': self.tmpdir,
             'soslog': self.soslog,
             'policy': self.policy,
+            'sysroot': self.sysroot,
             'verbosity': self.opts.verbosity,
             'xmlreport': self.xml_report,
             'cmdlineopts': self.opts,
             'config': self.config,
             'global_plugin_options': self.global_plugin_options,
-            }
+        }
 
     def get_temp_file(self):
         return self.tempfile_util.new()
@@ -675,8 +767,6 @@ class SoSReport(object):
         if self.opts.compression_type == 'auto':
             auto_archive = self.policy.get_preferred_archive()
             self.archive = auto_archive(archive_name, self.tmpdir)
-        elif self.opts.compression_type == 'zip':
-            self.archive = ZipFileArchive(archive_name, self.tmpdir)
         else:
             self.archive = TarFileArchive(archive_name, self.tmpdir)
         self.archive.set_debug(True if self.opts.debug else False)
@@ -729,10 +819,17 @@ class SoSReport(object):
             config_file = self.opts.config_file
         else:
             config_file = '/etc/sos.conf'
+
         try:
-            self.config.readfp(open(config_file))
-        except IOError:
-            pass
+            try:
+                with open(config_file) as f:
+                    self.config.readfp(f)
+            except (ParsingError, Error) as e:
+                raise exit('Failed to parse configuration '
+                           'file %s' % config_file)
+        except (OSError, IOError) as e:
+            raise exit('Unable to read configuration file %s '
+                       ': %s' % (config_file, e.args[1]))
 
     def _setup_logging(self):
         # main soslog
@@ -846,18 +943,27 @@ class SoSReport(object):
         self.plugin_names = deque()
         self.profiles = set()
         using_profiles = len(self.opts.profiles)
+        policy_classes = self.policy.valid_subclasses
+        extra_classes = []
+
+        if self.opts.experimental:
+            extra_classes.append(sos.plugins.ExperimentalPlugin)
+        valid_plugin_classes = tuple(policy_classes + extra_classes)
+        validate_plugin = self.policy.validate_plugin
+        remaining_profiles = list(self.opts.profiles)
+
         # validate and load plugins
         for plug in plugins:
             plugbase, ext = os.path.splitext(plug)
             try:
-                plugin_classes = import_plugin(
-                    plugbase, tuple(self.policy.valid_subclasses))
+                plugin_classes = import_plugin(plugbase, valid_plugin_classes)
                 if not len(plugin_classes):
                     # no valid plugin classes for this policy
                     continue
 
                 plugin_class = self.policy.match_plugin(plugin_classes)
-                if not self.policy.validate_plugin(plugin_class):
+                if not validate_plugin(plugin_class,
+                                       experimental=self.opts.experimental):
                     self.soslog.warning(
                         _("plugin %s does not validate, skipping") % plug)
                     if self.opts.verbosity > 0:
@@ -872,8 +978,6 @@ class SoSReport(object):
 
                 # plug-in is valid, let's decide whether run it or not
                 self.plugin_names.append(plugbase)
-                if hasattr(plugin_class, "profiles"):
-                    self.profiles.update(plugin_class.profiles)
 
                 in_profile = self._is_in_profile(plugin_class)
                 if not in_profile:
@@ -892,18 +996,31 @@ class SoSReport(object):
                     self._skip(plugin_class, _("optional"))
                     continue
 
+                # only add the plugin's profiles once we know it is usable
+                if hasattr(plugin_class, "profiles"):
+                    self.profiles.update(plugin_class.profiles)
+
                 # true when the null (empty) profile is active
                 default_profile = not using_profiles and in_profile
                 if self._is_not_specified(plugbase) and default_profile:
                     self._skip(plugin_class, _("not specified"))
                     continue
 
+                for i in plugin_class.profiles:
+                    if i in remaining_profiles:
+                        remaining_profiles.remove(i)
+
                 self._load(plugin_class)
             except Exception as e:
                 self.soslog.warning(_("plugin %s does not install, "
-                                    "skipping: %s") % (plug, e))
+                                      "skipping: %s") % (plug, e))
                 if self.raise_plugins:
                     raise
+        if len(remaining_profiles) > 0:
+            self.soslog.error(_("Unknown or inactive profile(s) provided:"
+                                " %s") % ", ".join(remaining_profiles))
+            self.list_profiles()
+            self._exit(1)
 
     def _set_all_options(self):
         if self.opts.usealloptions:
@@ -981,6 +1098,16 @@ class SoSReport(object):
                 self.all_options.append((plugin, plugin_name, optname,
                                          optparm))
 
+    def _report_profiles_and_plugins(self):
+        self.ui_log.info("")
+        if len(self.loaded_plugins):
+            self.ui_log.info(" %d profiles, %d plugins"
+                             % (len(self.profiles), len(self.loaded_plugins)))
+        else:
+            # no valid plugins for this profile
+            self.ui_log.info(" %d profiles" % len(self.profiles))
+        self.ui_log.info("")
+
     def list_plugins(self):
         if not self.loaded_plugins and not self.skipped_plugins:
             self.soslog.fatal(_("no valid plugins found"))
@@ -998,7 +1125,7 @@ class SoSReport(object):
 
         if self.skipped_plugins:
             self.ui_log.info(_("The following plugins are currently "
-                             "disabled:"))
+                               "disabled:"))
             self.ui_log.info("")
             for (plugname, plugclass, reason) in self.skipped_plugins:
                 self.ui_log.info(" %-20s %-14s %s" % (
@@ -1031,10 +1158,7 @@ class SoSReport(object):
         lines = _format_list("Profiles: ", profiles, indent=True)
         for line in lines:
             self.ui_log.info(" %s" % line)
-        self.ui_log.info("")
-        self.ui_log.info(" %d profiles, %d plugins"
-                         % (len(self.profiles), len(self.loaded_plugins)))
-        self.ui_log.info("")
+        self._report_profiles_and_plugins()
 
     def list_profiles(self):
         if not self.profiles:
@@ -1056,10 +1180,7 @@ class SoSReport(object):
             lines = _format_list("%-15s " % profile, plugins, indent=True)
             for line in lines:
                 self.ui_log.info(" %s" % line)
-        self.ui_log.info("")
-        self.ui_log.info(" %d profiles, %d plugins"
-                         % (len(profiles), len(self.loaded_plugins)))
-        self.ui_log.info("")
+        self._report_profiles_and_plugins()
 
     def batch(self):
         if self.opts.batch:
@@ -1073,14 +1194,20 @@ class SoSReport(object):
                 self.ui_log.info("")
                 self._exit()
 
-    def _log_plugin_exception(self, plugin_name):
-        self.soslog.error("%s\n%s" % (plugin_name, traceback.format_exc()))
+    def _log_plugin_exception(self, plugin, method):
+        trace = traceback.format_exc()
+        msg = "caught exception in plugin method"
+        plugin_err_log = "%s-plugin-errors.txt" % plugin
+        logpath = os.path.join(self.logdir, plugin_err_log)
+        self.soslog.error('%s "%s.%s()"' % (msg, plugin, method))
+        self.soslog.error('writing traceback to %s' % logpath)
+        self.archive.add_string("%s\n" % trace, logpath)
 
     def prework(self):
         self.policy.pre_work()
         try:
             self.ui_log.info(_(" Setting up archive ..."))
-            compression_methods = ('auto', 'zip', 'bzip2', 'gzip', 'xz')
+            compression_methods = ('auto', 'bzip2', 'gzip', 'xz')
             method = self.opts.compression_type
             if method not in compression_methods:
                 compression_list = ', '.join(compression_methods)
@@ -1093,10 +1220,13 @@ class SoSReport(object):
             self._make_archive_paths()
             return
         except (OSError, IOError) as e:
+            # we must not use the logging subsystem here as it is potentially
+            # in an inconsistent or unreliable state (e.g. an EROFS for the
+            # file system containing our temporary log files).
             if e.errno in fatal_fs_errors:
-                self.ui_log.error("")
-                self.ui_log.error(" %s while setting up archive" % e.strerror)
-                self.ui_log.error("")
+                print("")
+                print(" %s while setting up archive" % e.strerror)
+                print("")
             else:
                 raise e
         except Exception as e:
@@ -1124,11 +1254,13 @@ class SoSReport(object):
                                       % e.strerror)
                     self.ui_log.error("")
                     self._exit(1)
+                if self.raise_plugins:
+                    raise
+                self._log_plugin_exception(plugname, "setup")
             except:
                 if self.raise_plugins:
                     raise
-                else:
-                    self._log_plugin_exception(plugname)
+                self._log_plugin_exception(plugname, "setup")
 
     def version(self):
         """Fetch version information from all plugins and store in the report
@@ -1170,11 +1302,13 @@ class SoSReport(object):
                                       % e.strerror)
                     self.ui_log.error("")
                     self._exit(1)
+                if self.raise_plugins:
+                    raise
+                self._log_plugin_exception(plugname, "collect")
             except:
                 if self.raise_plugins:
                     raise
-                else:
-                    self._log_plugin_exception(plugname)
+                self._log_plugin_exception(plugname, "collect")
         self.ui_log.info("")
 
     def report(self):
@@ -1210,11 +1344,11 @@ class SoSReport(object):
 
             for f in plug.copied_files:
                 section.add(CopiedFile(name=f['srcpath'],
-                            href=".." + f['dstpath']))
+                                       href=".." + f['dstpath']))
 
             for cmd in plug.executed_commands:
                 section.add(Command(name=cmd['exe'], return_code=0,
-                            href="../" + cmd['file']))
+                                    href="../" + cmd['file']))
 
             for content, f in plug.copy_strings:
                 section.add(CreatedFile(name=f))
@@ -1222,7 +1356,8 @@ class SoSReport(object):
             report.add(section)
         try:
             fd = self.get_temp_file()
-            fd.write(str(PlainTextReport(report)))
+            output = PlainTextReport(report).unicode()
+            fd.write(output)
             fd.flush()
             self.archive.add_file(fd.name, dest=os.path.join('sos_reports',
                                                              'sos.txt'))
@@ -1317,38 +1452,124 @@ class SoSReport(object):
                                       % e.strerror)
                     self.ui_log.error("")
                     self._exit(1)
+                if self.raise_plugins:
+                    raise
+                self._log_plugin_exception(plugname, "postproc")
             except:
                 if self.raise_plugins:
                     raise
+                self._log_plugin_exception(plugname, "postproc")
+
+    def _create_checksum(self, archive, hash_name):
+        if not archive:
+            return False
+
+        archive_fp = open(archive, 'rb')
+        digest = hashlib.new(hash_name)
+        digest.update(archive_fp.read())
+        archive_fp.close()
+        return digest.hexdigest()
+
+    def _write_checksum(self, archive, hash_name, checksum):
+            # store checksum into file
+            fp = open(archive + "." + hash_name, "w")
+            if checksum:
+                fp.write(checksum + "\n")
+            fp.close()
 
     def final_work(self):
-        # this must come before archive creation to ensure that log
+        # This must come before archive creation to ensure that log
         # files are closed and cleaned up at exit.
+        #
+        # All subsequent terminal output must use print().
         self._finish_logging()
-        # package up the results for the support organization
+
+        archive = None    # archive path
+        directory = None  # report directory path (--build)
+
+        # package up and compress the results
         if not self.opts.build:
+            old_umask = os.umask(0o077)
             if not self.opts.quiet:
                 print(_("Creating compressed archive..."))
             # compression could fail for a number of reasons
             try:
-                final_filename = self.archive.finalize(
+                archive = self.archive.finalize(
                     self.opts.compression_type)
             except (OSError, IOError) as e:
                 if e.errno in fatal_fs_errors:
-                    self.ui_log.error("")
-                    self.ui_log.error(" %s while finalizing archive"
-                                      % e.strerror)
-                    self.ui_log.error("")
+                    print("")
+                    print(_(" %s while finalizing archive" % e.strerror))
+                    print("")
                     self._exit(1)
             except:
                 if self.opts.debug:
                     raise
                 else:
                     return False
+            finally:
+                os.umask(old_umask)
         else:
-            final_filename = self.archive.get_archive_path()
-        self.policy.display_results(final_filename, build=self.opts.build)
-        self.tempfile_util.clean()
+            # move the archive root out of the private tmp directory.
+            directory = self.archive.get_archive_path()
+            dir_name = os.path.basename(directory)
+            try:
+                final_dir = os.path.join(self.sys_tmp, dir_name)
+                os.rename(directory, final_dir)
+                directory = final_dir
+            except (OSError, IOError):
+                    print(_("Error moving directory: %s" % directory))
+                    return False
+
+        checksum = None
+
+        if not self.opts.build:
+            # compute and store the archive checksum
+            hash_name = self.policy.get_preferred_hash_name()
+            checksum = self._create_checksum(archive, hash_name)
+            self._write_checksum(archive, hash_name, checksum)
+
+            # output filename is in the private tmpdir - move it to the
+            # containing directory.
+            final_name = os.path.join(self.sys_tmp, os.path.basename(archive))
+
+            archive_hash = archive + "." + hash_name
+            final_hash = final_name + "." + hash_name
+
+            # move the archive and checksum file
+            try:
+                    os.rename(archive, final_name)
+                    archive = final_name
+            except (OSError, IOError):
+                    print(_("Error moving archive file: %s" % archive))
+                    return False
+
+            # There is a race in the creation of the final checksum file:
+            # since the archive has already been published and the checksum
+            # file name is predictable once the archive name is known a
+            # malicious user could attempt to create a symbolic link in order
+            # to misdirect writes to a file of the attacker's choosing.
+            #
+            # To mitigate this we write the checksum inside the private tmp
+            # directory and use an atomic rename that is guaranteed to either
+            # succeed or fail: at worst the move will fail and be reported to
+            # the user. The correct checksum value is still written to the
+            # terminal and nothing is written to a location under the control
+            # of the user creating the link.
+            try:
+                    os.rename(archive_hash, final_hash)
+            except (OSError, IOError):
+                    print(_("Error moving checksum file: %s" % archive_hash))
+                    return False
+
+        self.policy.display_results(archive, directory, checksum)
+
+        # clean up
+        if self.tempfile_util:
+            self.tempfile_util.clean()
+        if self.tmpdir:
+            rmtree(self.tmpdir)
+
         return True
 
     def verify_plugins(self):
@@ -1362,7 +1583,6 @@ class SoSReport(object):
 
     def execute(self):
         try:
-            self._setup_logging()
             self.policy.set_commons(self.get_commons())
             self.print_header()
             self.load_plugins()
@@ -1386,7 +1606,7 @@ class SoSReport(object):
             self.prework()
             self.setup()
             self.collect()
-            if not self.opts.report:
+            if not self.opts.noreport:
                 self.report()
                 self.html_report()
                 self.plain_report()
@@ -1394,12 +1614,21 @@ class SoSReport(object):
             self.version()
 
             return self.final_work()
-        except (SystemExit, KeyboardInterrupt):
-            if self.archive:
-                self.archive.cleanup()
-            if self.tempfile_util:
-                self.tempfile_util.clean()
-            return False
+
+        except (OSError, SystemExit, KeyboardInterrupt):
+            try:
+                # archive and tempfile cleanup may fail due to a fatal
+                # OSError exception (ENOSPC, EROFS etc.).
+                if self.archive:
+                    self.archive.cleanup()
+                if self.tempfile_util:
+                    self.tempfile_util.clean()
+                if self.tmpdir:
+                    rmtree(self.tmpdir)
+            except:
+                raise
+
+        return False
 
 
 def main(args):
@@ -1407,4 +1636,4 @@ def main(args):
     sos = SoSReport(args)
     sos.execute()
 
-# vim: et ts=4 sw=4
+# vim: set et ts=4 sw=4 :
